(defparameter *height* 20)
(defparameter *width* 60)
(defparameter *humans* nil)
(defparameter *enemys* nil)
(defparameter *init-member* 40)
(defparameter *init-enemys* 5)
(defparameter *life* 100)
(defparameter *place* nil)
(defparameter *energy* 50)
(defparameter *turn* 0)

(defstruct human
  x y age sex preg)

(defstruct enemy
  x y life)

(defun start ()
  (loop repeat *init-member*
	do (progn 
	     (let ((p (randput)))
	       (push (make-human :x (car p)
				 :y (cdr p)
				 :age (random (ash *life* -1))
				 :sex (random 2)
				 :preg nil)
		     *humans*)
	       (push (cons p 'h) *place*))))
  (loop repeat *init-enemys*
	do (progn
	     (let ((p (randput)))
	       (push (make-enemy :x (car p)
				 :y (cdr p)
				 :life 100)
		     *enemys*)
	       (push (cons p 'e) *place*))))
  (main-loop))

(defun randput ()
  (let ((p (cons (random *width*) (random *height*))))
    (if (placep p *place*)
	(randput)
      p)))

(defun placep (p lst)
  (if (null lst)
      nil
    (if (equal (caar lst) p)
	(cdar lst)
      (placep p (cdr lst)))))

(defun memberp (key lst)
  (if (null lst)
      nil
    (if (equal key (car lst))
	t
      (memberp key (cdr lst)))))

(defun main-loop ()
  (show-map)
  (fresh-line)
  (let ((str (read-line)))
    (cond ((equal str "quit") ())
	  (t (let ((x (parse-integer str :junk-allowed t)))
	       (if x
		   (loop
		     for i
		     below x
		     do (update-world)
		     if (zerop (mod i 1000))
		     do (princ #\.))
		 (update-world))
	       (main-loop))))))

(defun show-map ()
  (format t "~&~dN~&" *turn*)
  (princ "|")
  (loop for x
	below *width*
	do (princ #\*))
  (princ "|")
  (loop for y
	below *height*
	do (progn (fresh-line)
		  (princ "|")
		  (loop for x
			below *width*
			do (if (placep (cons x y) *place*)
			       (cond
				((eq (placep (cons x y) *place*) 'h)
				 (let ((h (search-place (cons x y) *humans*)))
				   (if (< (human-age h) 18)
				       (princ #\k)
				     (if (= (human-sex h) 0)
					 (if (< (human-age h) (ash *life* -1))
					     (princ #\f)
					   (princ #\F))
				       (if (< (human-age h) (ash *life* -1))
					   (princ #\m)
					 (princ #\M))))))
				((eq (placep (cons x y) *place*) 'e)
				 (princ #\e)))
			     (princ #\space)))
		  (princ "|")))
  (fresh-line)
  (princ "|")
  (loop for x
	below *width*
	do (princ #\*))
  (princ "|"))

(defun update-world ()
  (incf *turn*)
  (dolist (h *humans*)
    (incf (human-age h))
    (if (human-preg h)
	(bear (human-x h) (human-y h) h)
      (move h))
    (if (< *life* (human-age h))
	(progn (setf *place* (del-key (cons (cons (human-x h) (human-y h)) 'h) *place*))
	       (setf *humans* (del-key h *humans*)))))
  (dolist (e *enemys*)
    (decf (enemy-life e))
    (if (< (enemy-life e) 0)
	(progn (setf *place* (del-key (cons (cons (enemy-x e) (enemy-y e)) 'e) *place*))
	       (setf *enemys* (del-key e *enemys*)))
      (cond
       ((and (< 200 (enemy-life e))
	     (zerop (random 10)))
	(bear-enemy (enemy-x e) (enemy-y e))
	(decf (enemy-life e) 100))
       (t (move-enemy e))))))

(defun move-enemy (e)
  (let* ((x (1- (random 3)))
	 (y (1- (random 3)))
	 (px (+ (enemy-x e) x))
	 (py (+ (enemy-y e) y)))
    (if (not (and (= x 0)
		  (= y 0)))
	(cond
	 ((or (< px 0)
	      (< py 0)
	      (< *width* px)
	      (< *height* py))
	  (move-enemy e))
	 ((eq (placep (cons px py) *place*) 'e)
	  (move-enemy e))
	 ((eq (placep (cons px py) *place*) 'h)
	  (setf *place* (del-key (cons (cons px py) 'h) *place*))
	  (setf *humans* (del-key (search-place (cons px py) *humans*) *humans*))
	  (incf (enemy-life e) *energy*)
	  (setf *place* (del-key (cons (cons (enemy-x e) (enemy-y e)) 'e) *place*))
	  (setf (enemy-x e) px)
	  (setf (enemy-y e) py)
	  (push (cons (cons px py) 'e) *place*))
	 (t (setf *place* (del-key (cons (cons (enemy-x e) (enemy-y e)) 'e) *place*))
	    (setf (enemy-x e) px)
	    (setf (enemy-y e) py)
	    (push (cons (cons px py) 'e) *place*)))
      nil)))

(defun bear-enemy (x y)
  (cond
   ((not (placep (cons (1- x) y) *place*))
    (push (make-enemy :x (1- x)
		      :y y
		      :life 100)
	  *enemys*))
   ((not (placep (cons (1+ x) y) *place*))
    (push (make-enemy :x (1+ x)
		      :y y
		      :life 100)
	  *enemys*))
   ((not (placep (cons x (1- y)) *place*))
    (push (make-enemy :x x
		      :y (1- y)
		      :life 100)
	  *enemys*))
   ((not (placep (cons x (1+ y)) *place*))
    (push (make-enemy :x x
		      :y (1+ y)
		      :life 100)
	  *enemys*))))

(defun bear (x y h)
  (cond
   ((not (placep (cons (1- x) y) *place*))
    (push (make-human :x (1- x)
		      :y y
		      :age 0
		      :sex (random 2)
		      :preg nil)
	  *humans*)
    (setf (human-preg h) nil))
   ((not (placep (cons (1+ x) y) *place*))
    (push (make-human :x (1+ x)
		      :y y
		      :age 0
		      :sex (random 2)
		      :preg nil)
	  *humans*)
    (setf (human-preg h) nil))
   ((not (placep (cons x (1+ y)) *place*))
    (push (make-human :x x
		      :y (1+ y)
		      :age 0
		      :sex (random 2)
		      :preg nil)
	  *humans*)
    (setf (human-preg h) nil))
   ((not (placep (cons x (1- y)) *place*))
    (push (make-human :x x
		      :y (1- y)
		      :age 0
		      :sex (random 2)
		      :preg nil)
	  *humans*)
    (setf (human-preg h) nil))
   (t (human-preg h) nil)))

(defun search-place (p lst)
  (if (null lst)
      nil
    (if (and (= (car p) (human-x (car lst)))
	     (= (cdr p) (human-y (car lst))))
	(car lst)
      (search-place p (cdr lst)))))

(defun move (h)
  (labels ((mv (h)
	       (let* ((x (1- (random 3)))
		      (y (1- (random 3)))
		      (px (+ (human-x h) x))
		      (py (+ (human-y h) y)))
		 (if (not (and (= x 0)
			       (= y 0)))
		     (if (or (placep (cons px py) *place*)
			     (< px 0)
			     (< (1- *width*) px)
			     (< py 0)
			     (< (1- *height*) py))
			 (move h)
		       (progn (setf *place* (del-key (cons (cons (human-x h) (human-y h)) 'h) *place*))
			      (setf (human-x h) px)
			      (setf (human-y h) py)
			      (push (cons (cons px py) 'h) *place*)))))))
    (mv h))
  (if (< 18 (human-age h))
      (cond
       ((scheck h 1 0)
	(if (= (human-sex h) 0)
	    (setf (human-preg h) t)
	  (setf (human-preg (search-place (cons (1+ (human-x h)) (human-y h)) *humans*)) t)))
       ((scheck h -1 0)
	(if (= (human-sex h) 0)
	    (setf (human-preg h) t)
	  (setf (human-preg (search-place (cons (1- (human-x h)) (human-y h)) *humans*)) t)))
       ((scheck h 0 1)
	(if (= (human-sex h) 0)
	    (setf (human-preg h) t)
	  (setf (human-preg (search-place (cons (human-x h) (1+ (human-y h))) *humans*)) t)))
       ((scheck h 0 -1)
	(if (= (human-sex h) 0)
	    (setf (human-preg h) t)
	  (setf (human-preg (search-place (cons (human-x h) (1- (human-y h))) *humans*)) t))))))

(defun scheck (h x y)
  (if (memberp (cons (cons (+ (human-x h) x) (+ (human-y h) y)) 'h) *place*)
      (let ((n (search-place (cons (+ (human-x h) x) (+ (human-y h) y)) *humans*)))
	(if (and (/= (human-sex h) (human-sex n))
		 (< 18 (human-age h))
		 (< 18 (human-age n))
		 (< (human-age h) (ash *life* -1))
		 (< (human-age n) (ash *life* -1)))
	    t
	  nil))
    nil))

(defun del-key (p lst)
  (if (null lst)
      nil
    (if (equal p (car lst))
	(del-key p (cdr lst))
      (cons (car lst) (del-key p (cdr lst))))))

(start)